//*****************************************************************************
//* =====================
//*  HHSpring
//* =====================
//*
//* (Description)
//*    aa --> HH generator
//*
//* (Update Record)
//*    2008/07/09  K.Ikematsu   Derived from phssim/tau/FFStudy/FFSpring.cxx
//*
//*****************************************************************************

#include "JSFSteer.h"
#include "HHSpring.h"

#include "TRandom.h"

#include <sstream>
#include <iomanip>
//#define __DEBUG__

//*----------------------------------------------------------------------
//*     Numerical and Natural Constants
//*----------------------------------------------------------------------
#include "GENNumCon.h"

ClassImp(HHSpring)
ClassImp(HHSpringBuf)
ClassImp(HHBases)

extern "C" {
extern Complex_t amp_pp_(Double_t *s, Double_t *t, Double_t *u,
                         Double_t *mh, Double_t *mt, Double_t *mw,
                         Double_t *delkappa);
extern Complex_t amp_pm_(Double_t *s, Double_t *t, Double_t *u,
                         Double_t *mh, Double_t *mt, Double_t *mw,
                         Double_t *delkappa);
extern void ephcol_(Int_t *ifl, Double_t *xx, Double_t *ea1, Double_t *ea2,
                    Int_t *ihel1, Int_t *ihel2, Double_t *w);
};

//-----------------------------------------------------------------------------
// ==============================
//  class HHSpring
// ==============================
//_____________________________________________________________________________
// --------------------------
//  C-tor
// --------------------------
HHSpring::HHSpring(const char *name, const char *title, HHBases *bases)
          : JSFSpring(name, title, bases)
{
  fEventBuf = new HHSpringBuf("HHSpringBuf", "HHSpring event buffer", this);
  if (!bases) {
    SetBases(new HHBases);
  }
}

//_____________________________________________________________________________
// --------------------------
//  D-tor
// --------------------------
HHSpring::~HHSpring()
{
  //delete fEventBuf;
  //delete GetBases();
}


//-----------------------------------------------------------------------------
// ==============================
//  class HHSpringBuf
// ==============================
//_____________________________________________________________________________
// --------------------------
//  SetPartons
// --------------------------
Bool_t HHSpringBuf::SetPartons()
{
  TClonesArray &partons = *fPartons;
  HHBases      *bases   = (HHBases*)((HHSpring*)Module())->GetBases();

  // ----------------------------------------------
  //  Set 4-momenta of final state particles in CM
  // ----------------------------------------------
  fNparton = 2;

  static ANL4DVector *pv = 0;
  if (!pv) {
    pv = new ANL4DVector [fNparton];
  }
  pv[0] = bases->fP[0];
  pv[1] = bases->fP[1];

  // ----------------------------------------------
  //  Boost them to lab. frame
  // ----------------------------------------------

  fZBoost       = bases->GetZBoost();
  fEcmIP        = bases->GetEcmIP();
  fCosTheta     = bases->GetCosTheta();
  fPhi          = bases->GetPhi();
  Double_t elab = TMath::Sqrt(fEcmIP*fEcmIP + fZBoost*fZBoost);
  TVector3 boostv(0.,0.,fZBoost/elab);

  for (Int_t i=0; i<fNparton; i++) pv[i].Boost(boostv);

  // ----------------------------------------------
  //  Set final state parton infomation
  // ----------------------------------------------

  static TVector **qp = 0;
  if (!qp) {
    qp = new TVector* [fNparton];
    for (Int_t i=0; i<fNparton; i++) qp[i] = new TVector(4);
  }
  for (Int_t i=0; i<fNparton; i++) {
    TVector &q = *qp[i];
    q(0) = pv[i].E ();
    q(1) = pv[i].Px();
    q(2) = pv[i].Py();
    q(3) = pv[i].Pz();
  }
  Int_t    idh = 25;               // Higgs PID
  Double_t mh  = bases->GetMass(); // Higgs mass
  Double_t chh = 0.;               // Higgs charge

  //                               No.  PID  Mass  Charge      pv Nd 1st Mom hel col shower
  new (partons[0]) JSFSpringParton(  1, idh,   mh,    chh, *qp[0], 0,  0,  0,  0,  0,     0);
  new (partons[1]) JSFSpringParton(  2, idh,   mh,    chh, *qp[1], 0,  0,  0,  0,  0,     0);

  return kTRUE ;
}

//-----------------------------------------------------------------------------
// ==============================
//  class HHBases
// ==============================
//_____________________________________________________________________________
// --------------------------
//  C-tor
// --------------------------
HHBases::HHBases(const char *name, const char *title)
         : JSFBases   (name, title),
           fEcm       (300.),
           fMass      (120.),
           fDK        (0.),
           fZBoost    (0.),
           fEcmIP     (fEcm),
           fCosTheta  (0.),
           fPhi       (0.)
{
  //  Constructor of bases.  Default parameter should be initialized here
  //
  // --------------------------------------------
  //  Get parameters from jsf.conf, if specified
  // --------------------------------------------

  cout << "Init hhbases " << endl;

  using namespace std;

  stringstream ins(gJSF->Env()->GetValue("HHBases.Ecm","300.")); // E_cm
  ins >> fEcm;
  fEcmIP = fEcm;

  ins.clear();
  ins.str(gJSF->Env()->GetValue("HHBases.HMass","120."));        // higgs mass
  ins >> fMass;

  ins.clear();
  ins.str(gJSF->Env()->GetValue("HHBases.DeltaKappa","0."));     // delta kappa
  ins >> fDK;

  // --------------------------------------------
  //  Registor random numbers
  // --------------------------------------------
  //--
  //  Photon initial state helicity
  //--
  DefineVariable(fHelAA, 0., 1., 1, 1);
  DefineVariable(fR_Lum1, 0., 1., 1, 1);
  DefineVariable(fR_Lum2, 0., 1., 1, 1);

  //--
  //  cos(theta) and phi
  //--
  DefineVariable(fCosTheta , -1., +1., 1, 1);
  DefineVariable(fPhi      , 0., k2Pi, 0, 1);

  // --------------------------------------------
  //  Set Bases integration parameters
  // --------------------------------------------
  SetNoOfSample(10000);

  SetTuneValue (1.5);
  SetIteration1(0.05, 5);
  SetIteration2(0.05, 20);
}

//_____________________________________________________________________________
// --------------------------
//  D-tor
// --------------------------
HHBases::~HHBases()
{
}

//_____________________________________________________________________________
// --------------------------
//  Func
// --------------------------
Double_t HHBases::Func()
{
  //  Bases Integrand
  //

  Double_t bsWeight = 1.;  // Jacobian factor

  Double_t xx[3], ea1, ea2, w;
  Int_t    ifl;
  xx[0] = fR_Lum1;
  xx[1] = fR_Lum2;
  xx[2] = fHelAA;
  ephcol_(&ifl, xx, &ea1, &ea2, &fHelInitial[0], &fHelInitial[1], &w);
  ea1 *= fEcm/2;
  ea2 *= fEcm/2;
  fEcmIP   = 2.*TMath::Sqrt(ea1*ea2);     // ignore electron mass 
  Double_t sh = fEcmIP*fEcmIP;
  Double_t s0 = fEcm*fEcm;
#if 0
  cerr << "ea1 = " << ea1 << " ea2 = " << ea2 << " rsh = " << TMath::Sqrt(sh) << " w = " << w << endl;
#endif
#if 0
  bsWeight *= w * (s0/sh);
#else
  bsWeight *= w;
#endif

  fZBoost = ea1 - ea2; // P_z of the cm system

  // --------------------------------------------
  //  Select final state
  // --------------------------------------------
  Double_t weight = 1;
  Double_t mh     = GetMass();

  // --------------------------------------------
  //  Check if reduced Ecm enough for prod.
  // --------------------------------------------
  if (fEcmIP < 2*mh) {
    return 0.;
  }

  // --------------------------------------------
  //  Select helicity combination
  // --------------------------------------------
  //  Notice that spin average for photon is taken
  //  care of here
  SelectHelicities(weight);
  bsWeight *= weight;

  // --------------------------------------------
  //  Calcuate differential cross section
  // --------------------------------------------

  Double_t sigma = DSigmaDX();

  // --------------------------------------------
  //  Fill plots
  // --------------------------------------------

  H1Fill("h01", fEcmIP           , (bsWeight*sigma));
  H1Fill("h02", fCosTheta        , (bsWeight*sigma));
  H1Fill("h03", fPhi             , (bsWeight*sigma));
  H1Fill("h04", (Double_t)fJCombI, (bsWeight*sigma));

  return (bsWeight * sigma);
}

//_____________________________________________________________________________
// --------------------------
//  DSigmaDX
// --------------------------
Double_t HHBases::DSigmaDX()
{
  // --------------------------------------------
  //  Phase space
  // --------------------------------------------
  // fEcmIP, fCosTheta, fPhi);
  Double_t rs   = fEcmIP;
  Double_t cosx = fCosTheta;
  Double_t phix = fPhi;
  Double_t mh   = GetMass();
  Double_t mh2  = mh*mh;

  ANL4DVector qcm(rs, 0., 0., 0.);
  GENFrame    cmframe;

  GENPhase2 phaseCM(qcm, mh2, mh2, cmframe, cosx, phix, 0);
  Double_t betax = phaseCM.GetBetaBar();
  if (betax <= 0.) return 0.;
  fP[0] = phaseCM.GetFourMomentum(0);
  fP[1] = phaseCM.GetFourMomentum(1);

  Double_t eb     = rs/2.;
  Double_t beta_a = 1.;
  fK[0].SetXYZT(0., 0., eb, eb);
  fK[1].SetXYZT(0., 0.,-eb, eb);

  // --------------------------------------------
  //  Calcuate dihherential cross section
  // --------------------------------------------
  Double_t s      = rs * rs;
#ifdef __DEBUG__
  cerr << " fP[0] = (" 
       << fP[0].E () << ","
       << fP[0].Px() << ","
       << fP[0].Py() << ","
       << fP[0].Pz() << ")" << endl;
  cerr << " fP[1] = (" 
       << fP[1].E () << ","
       << fP[1].Px() << ","
       << fP[1].Py() << ","
       << fP[1].Pz() << ")" << endl;
  ANL4DVector pcm = fP[0] + fP[1];
  cerr << " pcm = (" 
       << pcm.E () << ","
       << pcm.Px() << ","
       << pcm.Py() << ","
       << pcm.Pz() << ")" << endl;
#endif

  // -------------------
  //  Amplitude squared
  // -------------------
  Complex_t amp  = AmpAAtoHH();
  Double_t  amp2 = (amp*conj(amp)).real();

  // -------------------
  //  Put them together
  // -------------------
  static const Int_t    kNbr  = 1;

  static const Double_t kFact = k2Pi/(TMath::Power(k4Pi,3*kNbr));

  Double_t identp = 0.5;                           // identical particle factor
  Double_t dPhase = kFact * betax;                 // phase space factor
  Double_t flux   = 1./(2.* s * beta_a);           // beam flux factor

  Double_t sigma  = identp * flux * amp2 * dPhase; // in [1/GeV^2]
           sigma *= kGeV2fb;                       // now in [fb]

  return sigma;
}

//_____________________________________________________________________________
// --------------------------
//  AmpAAtoHH()
// --------------------------
Complex_t HHBases::AmpAAtoHH()
{
  Double_t s  = (fK[0] + fK[1]).GetMass2();
  Double_t t  = (fP[0] - fK[0]).GetMass2();
  Double_t u  = (fP[1] - fK[0]).GetMass2();
  Double_t mh = GetMass();
  Double_t mt = 174.;
  Double_t mw = 80.;
  static const Double_t al_em = 1./128.;
  static const Double_t al_w  = 1./30.;

  Complex_t amp;
  if (fHelInitial[0] == fHelInitial[1]) amp = amp_pp_(&s, &t, &u, &mh, &mt, &mw, &fDK);
  else                                  amp = amp_pm_(&s, &t, &u, &mh, &mt, &mw, &fDK);
  amp *= al_em * al_w;

  return amp;
}

//_____________________________________________________________________________
// --------------------------
//  Userin
// --------------------------
void HHBases::Userin()
{
  // --------------------------------------------
  //  Define some plots
  // --------------------------------------------
  H1Init("h01", "Ecm"   , 50,     0., fEcm*1.1);
  H1Init("h02", "Costh" , 50,    -1.,      +1.);
  H1Init("h03", "Phi"   , 50,     0.,     k2Pi);
  H1Init("h04", "Helin" ,  4,     0.,       4.);
}

//_____________________________________________________________________________
// --------------------------
//  Userout
// --------------------------
void HHBases::Userout()
{
  cout << "End of HHBases----------------------------------- "  << endl
       << "Ecm                  = " << fEcm  << " [GeV]   "     << endl
       << "Mh                   = " << fMass << " [GeV]   "     << endl
       << "DK                   = " << fDK                      << endl
       << "Total Cross section  = " << GetEstimate()  << " +/- "
                                    << GetError()     << " [fb]"  << endl
       << "Number of iterations = " << GetNoOfIterate()           << endl;
}

//_____________________________________________________________________________
// --------------------------
//  SelectHelicities
// --------------------------
void HHBases::SelectHelicities(Double_t &weight)
{
   fJCombI =  2*( (fHelInitial[0] + 1)/2 ) + (fHelInitial[1] + 1)/2;
   weight = 1;
}
