#ifndef HHSPRING_H
#define HHSPRING_H
//*****************************************************************************
//* =====================
//*  HHSpring
//* =====================
//*
//* (Description)
//*    aa -> HH generator
//*
//* (Update Record)
//*    2008/07/09  K.Ikematsu	Derived from phssim/tau/FFStudy/FFSpring.h
//*
//*****************************************************************************
#include <complex>
typedef std::complex<Double_t> Complex_t;

#include "TNamed.h"
#include "JSFModule.h"
#include "JSFBases.h"
#include "JSFSpring.h"

#include "GENLib.h"

//_______________________________________________________________________
// =====================
//  class HHBases
// =====================
//-----------------------------------------------------------------------
class HHBases : public JSFBases {
friend class HHSpringBuf;
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  HHBases(const char *name  = "HHBases", 
          const char *title = "HH Bases");
  virtual ~HHBases();

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetEcm      ()           const { return fEcm;       }
  Double_t GetCosTheta ()           const { return fCosTheta;  }
  Double_t GetPhi      ()           const { return fPhi;       }
  Double_t GetZBoost   ()           const { return fZBoost;    }
  Double_t GetEcmIP    ()           const { return fEcmIP;     }
  Double_t GetMass     ()           const { return fMass;      }
  Double_t GetDK       ()           const { return fDK;        }

  void     SetEcmInit  (Double_t ecm       ) { fEcm     = ecm; }
  void     SetMass     (Double_t m         ) { fMass    = m;   }     
  void     SetDK       (Double_t dk        ) { fDK      = dk;  }     

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual void     Userin();   // Bases user initialization
  virtual void     Userout();  // Bases user output 

  Double_t Func();     // Bases integration function.

  // ----------------------
  //   Utility methods
  // ----------------------
private:
  void      SelectHelicities(Double_t &weight);

  Double_t  DSigmaDX     ();
  Complex_t AmpAAtoHH    ();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Job parameters
  // ----------------
  Double_t fEcm;            // Nominal Ecm
  Double_t fMass;           // Higss mass
  Double_t fDK;             // delta kappa

  // ----------------
  //  Event info
  // ----------------
  Double_t       fZBoost;         // p_z(cm) in lab frame
  Double_t       fEcmIP;          // Ecm
  Int_t          fHelInitial[2];  // initial state helicities
  Int_t          fJCombI;         // intial helicity combination
  ANL4DVector    fK[2];           // [0, 1] = [a, a]
  ANL4DVector    fP[2];           // [0, 1] = [h, h]

  // ----------------
  //  Integ. vars
  // ----------------
  Double_t       fHelAA;          //! initial state helicity combination
  Double_t       fR_Lum1;         //! luminosity spectrum for photon 1
  Double_t       fR_Lum2;         //!                     for photon 2
  Double_t       fCosTheta;       // cos(theta_x) in cm frame
  Double_t       fPhi;            // phi_x        in cm frame

  ClassDef(HHBases, 1) // Bases for aa -> hh process
};

class HHSpring;

//_______________________________________________________________________
// =====================
//  class HHSpringBuf
// =====================
//-----------------------------------------------------------------------
class HHSpringBuf : public JSFSpringBuf {
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  HHSpringBuf(const char *name   = "HHSpringBuf", 
              const char *title  = "HH Spring test event buffer",
	      HHSpring *spring = 0)
	     : JSFSpringBuf(name,title,(JSFSpring*)spring) {}
  virtual ~HHSpringBuf() {}

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetCosTheta ()           const { return fCosTheta;  }
  Double_t GetPhi      ()           const { return fPhi;       }
  Double_t GetZBoost   ()           const { return fZBoost;    }
  Double_t GetEcmIP    ()           const { return fEcmIP;     }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual Bool_t SetPartons();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Event info
  // ----------------
  Double_t fCosTheta;       // cos(theta_x) in cm  frame
  Double_t fPhi;            // phi_x        in cm  frame
  Double_t fZBoost;         // p_z(cm)      in lab frame
  Double_t fEcmIP;          // Ecm

  ClassDef(HHSpringBuf, 1)  // HHSpring event buffer
};

//_______________________________________________________________________
// =====================
//  class HHSpring
// =====================
//-----------------------------------------------------------------------
class HHSpring : public JSFSpring {
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
public:
   HHSpring(const char *name  = "HHSpring",
            const char *title = "HH Spring test",
            HHBases  *bases = 0);
   virtual ~HHSpring();

   ClassDef(HHSpring, 1)  // HHSpring class
};
#endif
