//*****************************************************************************
//* =====================
//*  WWSpring
//* =====================
//*
//* (Description)
//*    aa --> WW generator
//*
//* (Update Record)
//*    2008/12/28  K.Fujii  Original version.
//*
//*****************************************************************************

#include "JSFSteer.h"
#include "WWSpring.h"

#include <sstream>
#include <iomanip>
//#define __DEBUG__
//#define __NOLUMI__

//*----------------------------------------------------------------------
//*     Numerical and Natural Constants
//*----------------------------------------------------------------------
#include "GENNumCon.h"

ClassImp(WWSpring)
ClassImp(WWSpringBuf)
ClassImp(WWBases)

extern "C" {
extern Complex_t ampww_(Double_t *gwwa, Double_t *mw, Double_t *gamw, Double_t gwf[2],
	                Int_t ihel[2], Double_t kv[2][4],
			Int_t fhel[4], Double_t mf[4], Double_t pv[4][4]);
extern void ephcol_(Int_t *ifl, Double_t *xx, Double_t *ea1, Double_t *ea2,
                    Int_t *ihel1, Int_t *ihel2, Double_t *w);
};

//-----------------------------------------------------------------------------
// ==============================
//  class WWSpring
// ==============================
//_____________________________________________________________________________
// --------------------------
//  C-tor
// --------------------------
WWSpring::WWSpring(const char *name, const char *title, WWBases *bases)
          : JSFSpring(name, title, bases)
{
  fEventBuf = new WWSpringBuf("WWSpringBuf", "WWSpring event buffer", this);
  if (!bases) {
    SetBases(new WWBases);
  }
}

//_____________________________________________________________________________
// --------------------------
//  D-tor
// --------------------------
WWSpring::~WWSpring()
{
  //delete fEventBuf;
  //delete GetBases();
}


//-----------------------------------------------------------------------------
// ==============================
//  class WWSpringBuf
// ==============================
//_____________________________________________________________________________
// --------------------------
//  SetPartons
// --------------------------
Bool_t WWSpringBuf::SetPartons()
{
  TClonesArray &partons = *fPartons;
  WWBases      *bases   = static_cast<WWBases *>(static_cast<WWSpring *>(Module())->GetBases());

  // ----------------------------------------------
  //  Set 4-momenta of final state particles in CM
  // ----------------------------------------------
  fNparton = 6;

  static ANL4DVector *pv = 0;
  if (!pv) {
    pv = new ANL4DVector [fNparton];
  }
  pv[2] = bases->fP[0];
  pv[3] = bases->fP[1];
  pv[4] = bases->fP[2];
  pv[5] = bases->fP[3];
  pv[0] = pv[2] + pv[3];
  pv[1] = pv[4] + pv[5];

  // ----------------------------------------------
  //  Boost them to lab. frame
  // ----------------------------------------------

  fZBoost       = bases->GetZBoost();
  fEcmIP        = bases->GetEcmIP();
  fQ2WW         = bases->GetQ2WW();
  fCosTheta     = bases->GetCosTheta();
  fPhi          = bases->GetPhi();
  fQ2WM         = bases->GetQ2WM();
  fCosThetaFM   = bases->GetCosThetaFM();
  fPhiFM        = bases->GetPhiFM();
  fQ2WP         = bases->GetQ2WP();
  fCosThetaFP   = bases->GetCosThetaFP();
  fPhiFP        = bases->GetPhiFP();

  Double_t elab = TMath::Sqrt(fEcmIP*fEcmIP + fZBoost*fZBoost);
  TVector3 boostv(0.,0.,fZBoost/elab);

  for (Int_t i=0; i<fNparton; i++) pv[i].Boost(boostv);

  // ----------------------------------------------
  //  Set final state parton infomation
  // ----------------------------------------------

  static TVector **qp = 0;
  if (!qp) {
    qp = new TVector* [fNparton];
    for (Int_t i=0; i<fNparton; i++) qp[i] = new TVector(4);
  }
  for (Int_t i=0; i<fNparton; i++) {
    TVector &q = *qp[i];
    q(0) = pv[i].E ();
    q(1) = pv[i].Px();
    q(2) = pv[i].Py();
    q(3) = pv[i].Pz();
  }
  Int_t    idw    = 24;               // W+ PID
  Double_t chw    = +1.;              // W+ charge
  Double_t rq2wm  = pv[0].GetMass();
  Double_t rq2wp  = pv[1].GetMass();

  Int_t    id1    = bases->f1Ptr->GetPID();
  Int_t    id2    = bases->f2Ptr->GetPID();
  Double_t m1     = bases->f1Ptr->GetMass();
  Double_t m2     = bases->f2Ptr->GetMass();
  Int_t    hel1   = bases->fHelFinal[0];
  Int_t    hel2   = bases->fHelFinal[1];
  Double_t chg1   = bases->f1Ptr->GetCharge();
  Double_t chg2   = bases->f2Ptr->GetCharge();
  Double_t colorm = bases->f1Ptr->GetColor();
  Int_t    islevm = colorm > 1. ? 101 : 0;
  Int_t    icfm   = 1;

  Int_t    id3    = bases->f3Ptr->GetPID();
  Int_t    id4    = bases->f4Ptr->GetPID();
  Double_t m3     = bases->f3Ptr->GetMass();
  Double_t m4     = bases->f4Ptr->GetMass();
  Int_t    hel3   = bases->fHelFinal[2];
  Int_t    hel4   = bases->fHelFinal[3];
  Double_t chg3   = bases->f3Ptr->GetCharge();
  Double_t chg4   = bases->f4Ptr->GetCharge();
  Double_t colorp = bases->f3Ptr->GetColor();
  Int_t    islevp = colorp > 1. ? 201 : 0;
  Int_t    icfp   = 2;

  //                               No.  PID  Mass  Charge    pv  Nd 1st Mom  hel  col shower
  new (partons[0]) JSFSpringParton(  1,-idw, rq2wm, -chw, *qp[0], 2,  3,  0,   0,   0,     0);
  new (partons[1]) JSFSpringParton(  2, idw, rq2wp,  chw, *qp[1], 2,  5,  0,   0,   0,     0);
  new (partons[2]) JSFSpringParton(  3,-id1,    m1,-chg1, *qp[2], 0,  0,  1,hel1,icfm,islevm);
  new (partons[3]) JSFSpringParton(  4,-id2,    m2,-chg2, *qp[3], 0,  0,  1,hel2,icfm,islevm);
  new (partons[4]) JSFSpringParton(  5, id3,    m3, chg3, *qp[4], 0,  0,  2,hel3,icfp,islevp);
  new (partons[5]) JSFSpringParton(  6, id4,    m4, chg4, *qp[5], 0,  0,  2,hel4,icfp,islevp);

  return kTRUE ;
}

//-----------------------------------------------------------------------------
// ==============================
//  class WWBases
// ==============================
//_____________________________________________________________________________
// --------------------------
//  C-tor
// --------------------------
WWBases::WWBases(const char *name, const char *title)
         : JSFBases   (name, title),
           fEcm       (300.),
           fWmModesLo ( 1),
           fWmModesHi (12),
           fWpModesLo ( 1),
           fWpModesHi (12),
           fWMBosonPtr( 0),
           fWPBosonPtr( 0),
           fZBoost    (0.),
           fEcmIP     (fEcm)
{
  //  Constructor of bases.  Default parameter should be initialized here
  //
  // --------------------------------------------
  //  Get parameters from jsf.conf, if specified
  // --------------------------------------------

  cout << "Init wwbases " << endl;

  using namespace std;

  stringstream ins(gJSF->Env()->GetValue("WWBases.Ecm","300.")); // E_cm
  ins >> fEcm;
  fEcmIP = fEcm;

  ins.clear();
  ins.str(gJSF->Env()->GetValue("WWBases.WmModesLo","1"));     // W- decay mode lo
  ins >> fWmModesLo;

  ins.clear();
  ins.str(gJSF->Env()->GetValue("WWBases.WmModesHi","12"));    // W- decay mode hi
  ins >> fWmModesHi;

  ins.clear();
  ins.str(gJSF->Env()->GetValue("WWBases.WpModesLo","1"));     // W+ decay mode lo
  ins >> fWpModesLo;

  ins.clear();
  ins.str(gJSF->Env()->GetValue("WWBases.WpModesHi","12"));    // W+ decay mode hi
  ins >> fWpModesHi;

  // --------------------------------------------
  //  Registor random numbers
  // --------------------------------------------
  //--
  //  Photon initial state helicity
  //--
  DefineVariable(fHelCombFinal, 0., 1., 1, 1);
  DefineVariable(fHelAA       , 0., 1., 1, 1);
  DefineVariable(fR_Lum1      , 0., 1., 1, 1);
  DefineVariable(fR_Lum2      , 0., 1., 1, 1);
  //--
  //  W- and W+ decay modes
  //--
#ifndef __NODECAY__
  DefineVariable(fWMDecayMode , 0., 1., 0, 1);
  DefineVariable(fWPDecayMode , 0., 1., 0, 1);
#else
  DefineVariable(fWMDecayMode , 0., 1., 1, 1);
  DefineVariable(fWPDecayMode , 0., 1., 1, 1);
#endif
  //--
  //  W- and W+ invariant masses
  //--
  DefineVariable(fXQ2WM       , 0., 1., 1, 1);
  DefineVariable(fXQ2WP       , 0., 1., 1, 1);
  //--
  //  cos(theta) and phi
  //--
  DefineVariable(fCosTheta  , -1., +1., 1, 1);
  DefineVariable(fPhi       , 0., k2Pi, 0, 1);
  DefineVariable(fCosThetaFM, -1., +1., 1, 1);
  DefineVariable(fPhiFM     , 0., k2Pi, 0, 1);
  DefineVariable(fCosThetaFP, -1., +1., 1, 1);
  DefineVariable(fPhiFP     , 0., k2Pi, 0, 1);

  // --------------------------------------------
  //  Set Bases integration parameters
  // --------------------------------------------
  SetNoOfSample(800000);

  SetTuneValue (1.5);
  SetIteration1(0.05, 200);
  SetIteration2(0.01, 500);

}
// --------------------------
//  D-tor
// --------------------------
WWBases::~WWBases()
{
  delete fWMBosonPtr;
  delete fWPBosonPtr;
}

//_____________________________________________________________________________
// --------------------------
//  Func
// --------------------------
Double_t WWBases::Func()
{
  //  Bases Integrand
  //

  Double_t bsWeight = 1.;  // Jacobian factor
  Double_t xx[3], ea1, ea2, w;
  Int_t    ifl;
  xx[0] = fR_Lum1;
  xx[1] = fR_Lum2;
  xx[2] = fHelAA;
#ifndef __NOLUMI__
  ephcol_(&ifl, xx, &ea1, &ea2, &fHelInitial[0], &fHelInitial[1], &w);
#else
  ea1 = 0.8;
  ea2 = 0.8;
  w   = 1.;
  fHelInitial[0] = -1;
  fHelInitial[1] = -1;
#endif
  ea1 *= fEcm/2;
  ea2 *= fEcm/2;
  fEcmIP   = 2.*TMath::Sqrt(ea1*ea2);     // ignore electron mass 
  bsWeight *= w;

  fZBoost = ea1 - ea2; // P_z of the cm system

  // --------------------------------------------
  //  Select final state
  // --------------------------------------------
  Double_t weight = 1;

  fWMModePtr = fWMBosonPtr->PickMode(fWMDecayMode, weight, fWMMode);
#ifndef __NODECAY__
  bsWeight *= weight;
#else
  if (!fWMModePtr) {
     cerr << " fWMMode = " << fWMMode 
          << " fWMDecayMode = " << fWMDecayMode << endl;
  }
#endif
  f1Ptr = static_cast<GENPDTEntry *>(fWMModePtr->At(0));
  f2Ptr = static_cast<GENPDTEntry *>(fWMModePtr->At(1));
  Double_t m1   = f1Ptr->GetMass();
  Double_t m2   = f2Ptr->GetMass();

  fWPModePtr = fWPBosonPtr->PickMode(fWPDecayMode, weight, fWPMode);
#ifndef __NODECAY__
  bsWeight *= weight;
#else
  if (!fWPModePtr) {
     cerr << " fWPMode = " << fWPMode
          << " fWPDecayMode = " << fWPDecayMode << endl;
  }
#endif
  f3Ptr = static_cast<GENPDTEntry *>(fWPModePtr->At(0));
  f4Ptr = static_cast<GENPDTEntry *>(fWPModePtr->At(1));
  Double_t m3   = f3Ptr->GetMass();
  Double_t m4   = f4Ptr->GetMass();

  // --------------------------------------------
  //  Check if reduced Ecm enough for prod.
  // --------------------------------------------
  if (fEcmIP < m1 + m2 + m3 + m4) {
    return 0.;
  }

  // --------------------------------------------
  //  Select helicity combination
  // --------------------------------------------
  //  Notice that spin average for photon is taken
  //  care of here
  SelectHelicities(weight);
  bsWeight *= weight;

  // --------------------------------------------
  //  Decide Q^2 of internal lines
  // --------------------------------------------
  fQ2WW = fEcmIP*fEcmIP;
  Double_t rs   = fEcmIP;
#ifndef __ZEROWIDTH__
  Double_t qmin = m1 + m2;
  Double_t qmax = rs - (m3 + m4);
  fQ2WM = fWMBosonPtr->GetQ2BW(qmin, qmax, fXQ2WM, weight);
  Double_t qwm = TMath::Sqrt(fQ2WM);
#else
  fQ2WM  = TMath::Power(fWMBosonPtr->GetMass(),2);
  weight = kPi*fWMBosonPtr->GetMass()*fWMBosonPtr->GetWidth();
#ifdef __NODECAY__
  weight = 1.;
#endif
#endif
  bsWeight *= weight;
#ifndef __ZEROWIDTH__
  qmin  = m3 + m4;
  qmax  = rs - qwm;
  fQ2WP = fWPBosonPtr->GetQ2BW(qmin, qmax, fXQ2WP, weight);
#else
  fQ2WP  = TMath::Power(fWPBosonPtr->GetMass(),2);
  weight = kPi*fWPBosonPtr->GetMass()*fWPBosonPtr->GetWidth();
#ifdef __NODECAY__
  weight = 1.;
#endif
#endif
  bsWeight *= weight;

  // --------------------------------------------
  //  Calcuate differential cross section
  // --------------------------------------------
  GENBranch wmbranch(fQ2WM, fCosThetaFM, fPhiFM, m1*m1    , m2*m2          );
  GENBranch wpbranch(fQ2WP, fCosThetaFP, fPhiFP, m3*m3    , m4*m4          );
  GENBranch cmbranch(fQ2WW, fCosTheta  , fPhi  , &wmbranch, &wpbranch      );

  Double_t sigma = DSigmaDX(cmbranch);

  // --------------------------------------------
  //  Fill plots
  // --------------------------------------------

  H1Fill("h01", fEcmIP           , (bsWeight*sigma));
  H1Fill("h02", fCosTheta        , (bsWeight*sigma));
  H1Fill("h03", fPhi             , (bsWeight*sigma));
  H1Fill("h04", (Double_t)fJCombI, (bsWeight*sigma));

  return (bsWeight * sigma);
}

//_____________________________________________________________________________
// --------------------------
//  DSigmaDX
// --------------------------
Double_t WWBases::DSigmaDX(GENBranch &cmbranch)
{
  // --------------------------------------------
  //  Phase space
  // --------------------------------------------
  Double_t rs = TMath::Sqrt(cmbranch.GetQ2());
  ANL4DVector qcm(rs, 0., 0., 0.);
  GENFrame    cmframe;
  
  // CM -> X- X+
  Double_t q2wm  = cmbranch.GetM12();
  Double_t q2wp  = cmbranch.GetM22();
  Double_t coswm = cmbranch.GetCosTheta();
  Double_t phiwm = cmbranch.GetPhi     ();
  GENPhase2 phaseCM(qcm, q2wm, q2wp, cmframe, coswm, phiwm, 0);
  ANL4DVector pwm = phaseCM.GetFourMomentum(0);
  ANL4DVector pwp = phaseCM.GetFourMomentum(1);
  Double_t betaw  = phaseCM.GetBetaBar();
  if (betaw <= 0.) return 0.;

  // W- -> fub + fd
  GENBranch &wmbranch = * cmbranch.GetBranchPtr(0);
  Double_t cosfm   = wmbranch.GetCosTheta();
  Double_t phifm   = wmbranch.GetPhi     ();
  Double_t m12     = wmbranch.GetM12();
  Double_t m22     = wmbranch.GetM22();

  GENPhase2 phaseWm(pwm, m12, m22, cmframe, cosfm, phifm, 1);
  fP[0] = phaseWm.GetFourMomentum(0);
  fP[1] = phaseWm.GetFourMomentum(1);
  fM[0] = TMath::Sqrt(m12);
  fM[1] = TMath::Sqrt(m22);
  Double_t betafm = phaseWm.GetBetaBar();
  if (betafm <= 0.) return 0.;

  // W+ -> fu + fdb
  GENBranch &wpbranch = * cmbranch.GetBranchPtr(1);
  Double_t cosfp   = wpbranch.GetCosTheta();
  Double_t phifp   = wpbranch.GetPhi     ();
  Double_t m32     = wpbranch.GetM12();
  Double_t m42     = wpbranch.GetM22();

  GENPhase2 phaseWp(pwp, m32, m42, cmframe, cosfp, phifp, 1);
  fP[2] = phaseWp.GetFourMomentum(0);
  fP[3] = phaseWp.GetFourMomentum(1);
  fM[2] = TMath::Sqrt(m32);
  fM[3] = TMath::Sqrt(m42);
  Double_t betafp = phaseWp.GetBetaBar();
  if (betafp <= 0.) return 0.;

  Double_t eb     = rs/2.;
  Double_t beta_a = 1.;
  fK[0].SetXYZT(0., 0., eb, eb);
  fK[1].SetXYZT(0., 0.,-eb, eb);

  // --------------------------------------------
  //  Calcuate diwwerential cross section
  // --------------------------------------------
  Double_t s      = rs * rs;
#ifdef __DEBUG__
#if 0
  cerr << " fP[0] = (" 
       << fP[0].E () << ","
       << fP[0].Px() << ","
       << fP[0].Py() << ","
       << fP[0].Pz() << ")" << endl;
  cerr << " fP[1] = (" 
       << fP[1].E () << ","
       << fP[1].Px() << ","
       << fP[1].Py() << ","
       << fP[1].Pz() << ")" << endl;
  cerr << " fP[2] = (" 
       << fP[2].E () << ","
       << fP[2].Px() << ","
       << fP[2].Py() << ","
       << fP[2].Pz() << ")" << endl;
  cerr << " fP[3] = (" 
       << fP[3].E () << ","
       << fP[3].Px() << ","
       << fP[3].Py() << ","
       << fP[3].Pz() << ")" << endl;
#endif
  ANL4DVector qwm = fP[0] + fP[1];
  ANL4DVector qwp = fP[2] + fP[3];
  cerr << " qwm = (" 
       << qwm.E () << ","
       << qwm.Px() << ","
       << qwm.Py() << ","
       << qwm.Pz() << ")" << endl;
  cerr << " qwp = (" 
       << qwp.E () << ","
       << qwp.Px() << ","
       << qwp.Py() << ","
       << qwp.Pz() << ")" << endl;
  cerr << " mw- = " << qwm.GetMass() << " mw+ = " << qwp.GetMass() << endl;
  ANL4DVector pcm = qwm + qwp;
  cerr << " pcm = (" 
       << pcm.E () << ","
       << pcm.Px() << ","
       << pcm.Py() << ","
       << pcm.Pz() << ")" << endl;
#endif

  // -------------------
  //  Amplitude squared
  // -------------------
  Complex_t amp  = AmpAAtoWW();
  Double_t  amp2 = (amp*conj(amp)).real();

  // -------------------
  //  Put them together
  // -------------------
#ifndef __NODECAY__
  static const Int_t    kNbr  = 3;
  static const Double_t kFact = k2Pi/(TMath::Power(k4Pi,3*kNbr));
#else
  static const Double_t kFact = k2Pi/(TMath::Power(k4Pi,3+2));
  betafm = 1.;
  betafp = 1.;
#endif

  Double_t identp = 1.0;                           // identical particle factor
  Double_t dPhase = kFact * betaw * betafm*betafp; // phase space factor
  Double_t flux   = 1./(2.* s * beta_a);           // beam flux factor

  Double_t sigma  = identp * flux * amp2 * dPhase; // in [1/GeV^2]
           sigma *= kGeV2fb;                       // now in [fb]

  return sigma;
}

// --------------------------
//  AmpAAtoWW()
// --------------------------
Complex_t WWBases::AmpAAtoWW()
{
  Double_t mw   = fWMBosonPtr->GetMass ();
  Double_t gamw = fWMBosonPtr->GetWidth();
  Double_t gwwa = kGe;
  Double_t gwf[2];
  gwf[0] = -kGw*kSqh;
  gwf[1] = 0.;

  Double_t kv  [2][4];
  for (Int_t i=0; i<2; i++) {
    kv[i][0] = fK[i].E();
    kv[i][1] = fK[i].Px();
    kv[i][2] = fK[i].Py();
    kv[i][3] = fK[i].Pz();
  }
  Double_t pv  [4][4];
  for (Int_t i=0; i<4; i++) {
    pv[i][0] = fP[i].E();
    pv[i][1] = fP[i].Px();
    pv[i][2] = fP[i].Py();
    pv[i][3] = fP[i].Pz();
  }

  Complex_t amp = ampww_(&gwwa, &mw, &gamw, gwf,
                         fHelInitial, kv,
                         fHelFinal, fM, pv);
#ifndef __NODECAY__
  Double_t  color = TMath::Sqrt(f1Ptr->GetColor() * f3Ptr->GetColor());
  Int_t     ig1   = f1Ptr->GetGenNo() - 1;
  Int_t     ig2   = f2Ptr->GetGenNo() - 1;
  Int_t     ig3   = f3Ptr->GetGenNo() - 1;
  Int_t     ig4   = f4Ptr->GetGenNo() - 1;
  Double_t  mix   = kVkm[ig1][ig2]*kVkm[ig3][ig4];
#else
  Double_t  color = 1.;
  Double_t  mix   = 1.;
#endif
  amp *= color * mix;

  return amp;
}

//_____________________________________________________________________________
// --------------------------
//  Userin
// --------------------------
void WWBases::Userin()
{
  // --------------------------------------------
  //  Initialize decay table
  // --------------------------------------------
  if (!fWMBosonPtr) fWMBosonPtr = new GENPDTWBoson();
  for (Int_t m=1; m<=fWMBosonPtr->GetEntries(); m++) {
     GENDecayMode *mp = fWMBosonPtr->GetMode(m); 
     if (mp && (m<fWmModesLo || m>fWmModesHi)) {
        mp->Lock();
     }
  }
  fWMBosonPtr->DebugPrint();
  if (!fWPBosonPtr) fWPBosonPtr = new GENPDTWBoson();
  for (Int_t m=1; m<=fWPBosonPtr->GetEntries(); m++) {
     GENDecayMode *mp = fWPBosonPtr->GetMode(m); 
     if (mp && (m<fWpModesLo || m>fWpModesHi)) {
        mp->Lock();
     }
  }
  fWPBosonPtr->DebugPrint();

  // --------------------------------------------
  //  Define some plots
  // --------------------------------------------
  H1Init("h01", "Ecm"   , 100,     0., fEcm*1.1);
  H1Init("h02", "Costh" , 100,    -1.,      +1.);
  H1Init("h03", "Phi"   , 100,     0.,     k2Pi);
  H1Init("h04", "Helin" ,   4,     0.,       4.);
}

//_____________________________________________________________________________
// --------------------------
//  Userout
// --------------------------
void WWBases::Userout()
{
  cout << "End of WWBases----------------------------------- "  << endl
       << "Ecm                  = " << fEcm  << " [GeV]   "     << endl
       << "Total Cross section  = " << GetEstimate()  << " +/- "
                                    << GetError()     << " [fb]"<< endl
       << "Number of iterations = " << GetNoOfIterate()         << endl;
}

//_____________________________________________________________________________
// --------------------------
//  SelectHelicities
// --------------------------
void WWBases::SelectHelicities(Double_t &weight)
{
#ifndef __NODECAY__
   static const Int_t kNf = 1;
   static const Int_t kFHelComb[kNf][4] = {{+1, -1, -1, +1}};
#else
   static const Int_t kNf = 9;
   static const Int_t kFHelComb[kNf][4] = {{-1, -1, -1, +1},
                                           {-1,  0, -1, +1},
                                           {-1, +1, -1, +1},
                                           { 0, -1, -1, +1},
                                           { 0,  0, -1, +1},
                                           { 0, +1, -1, +1},
                                           {+1, -1, -1, +1},
                                           {+1,  0, -1, +1},
                                           {+1, +1, -1, +1}};
#endif
   fJCombI =  2*( (fHelInitial[0] + 1)/2 ) + (fHelInitial[1] + 1)/2;

   fJCombF = (Int_t)(fHelCombFinal*kNf);
   fJCombF = TMath::Min(fJCombF, kNf-1);
   fHelFinal  [0] = kFHelComb[fJCombF][0];
   fHelFinal  [1] = kFHelComb[fJCombF][1];
   fHelFinal  [2] = kFHelComb[fJCombF][2];
   fHelFinal  [3] = kFHelComb[fJCombF][3];
   weight = kNf;
}
