#ifndef WWSPRING_H
#define WWSPRING_H
//*****************************************************************************
//* =====================
//*  WWSpring
//* =====================
//*
//* (Description)
//*    aa -> WW generator
//*
//* (Update Record)
//*    2008/12/28  K.Fujii  Original version.
//*
//*****************************************************************************
#include <complex>
typedef std::complex<Double_t> Complex_t;

#include "TNamed.h"
#include "JSFModule.h"
#include "JSFBases.h"
#include "JSFSpring.h"

#include "GENLib.h"

//_______________________________________________________________________
// =====================
//  class WWBases
// =====================
//-----------------------------------------------------------------------
class WWBases : public JSFBases {
friend class WWSpringBuf;
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  WWBases(const char *name  = "WWBases", 
          const char *title = "WW Bases");
  virtual ~WWBases();

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetEcm       ()           const { return fEcm;        }
  Double_t GetQ2WW      ()           const { return fQ2WW;       }
  Double_t GetCosTheta  ()           const { return fCosTheta;   }
  Double_t GetPhi       ()           const { return fPhi;        }
  Double_t GetQ2WM      ()           const { return fQ2WM;       }
  Double_t GetCosThetaFM()           const { return fCosThetaFM; }
  Double_t GetPhiFM     ()           const { return fPhiFM;      }
  Double_t GetQ2WP      ()           const { return fQ2WP;       }
  Double_t GetCosThetaFP()           const { return fCosThetaFP; }
  Double_t GetPhiFP     ()           const { return fPhiFP;      }
  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }

  void     SetEcmInit  (Double_t ecm       ) { fEcm     = ecm; }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual void     Userin();   // Bases user initialization
  virtual void     Userout();  // Bases user output 

  Double_t Func();     // Bases integration function.

  // ----------------------
  //   Utility methods
  // ----------------------
private:
  void      SelectHelicities(Double_t &weight);

  Double_t  DSigmaDX     (GENBranch &cmbranch);
  Complex_t AmpAAtoWW    ();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Job parameters
  // ----------------
  Double_t fEcm;            // Nominal Ecm
  Int_t    fWmModesLo;      // W- decay mode lo;
  Int_t    fWmModesHi;      // W- decay mode hi;
  Int_t    fWpModesLo;      // W+ decay mode lo;
  Int_t    fWpModesHi;      // W+ decay mode hi;

  // ----------------
  //  Particle Data
  // ----------------
  GENPDTWBoson *fWMBosonPtr;      //! PD table entry of "WM"
  GENPDTWBoson *fWPBosonPtr;      //! PD table entry of "WP"

  // ----------------
  //  Event info
  // ----------------
  Double_t       fZBoost;         // p_z(cm) in lab frame
  Double_t       fEcmIP;          // Ecm
  Double_t       fQ2WW;           // q^2 of WW system
  Double_t       fQ2WM;           // q^2 of final state W-
  Double_t       fQ2WP;           // q^2 of final state W+
  GENDecayMode  *fWMModePtr;      // pointer to W- decay mode
  GENPDTEntry   *f1Ptr;           // point to 1st W- daughter
  GENPDTEntry   *f2Ptr;           // point to 2nd W- daughter
  GENDecayMode  *fWPModePtr;      // pointer to W+ decay mode
  GENPDTEntry   *f3Ptr;           // point to 1st W+ daughter
  GENPDTEntry   *f4Ptr;           // point to 2nd W+ daughter
  Int_t          fHelInitial[2];  // initial state helicities
  Int_t          fHelFinal  [4];  // final   state helicities
  Int_t          fJCombI;         // intial helicity combination
  Int_t          fJCombF;         // final  helicity combination
  Int_t          fWMMode;         // W- decay mode
  Int_t          fWPMode;         // W+ decay mode
  ANL4DVector    fK[2];           // [0, 1] = [a, a]
  ANL4DVector    fP[4];           // [0, 1, 2, 3] = [fd, fub, fdb, fu]
  Double_t       fM[4];           // [0,1,2,3] = [mfd, mfub, mfdb, mfu]

  // ----------------
  //  Integ. vars
  // ----------------
  Double_t       fHelAA;          //! initial state helicity combination
  Double_t       fR_Lum1;         //! luminosity spectrum for photon 1
  Double_t       fR_Lum2;         //!                     for photon 2
  Double_t       fHelCombFinal;   // final   state helicity combination
  Double_t       fWMDecayMode;    // decay mode selector for W-
  Double_t       fWPDecayMode;    // decay mode selector for W+
  Double_t       fCosTheta;       // cos(theta_W-)  in cm frame
  Double_t       fPhi;            // phi_W-         in cm frame
  Double_t       fCosThetaFM;     // cos(theta_fd)  in W- frame
  Double_t       fPhiFM;          // phi_fd         in W- frame
  Double_t       fXQ2WM;          // q^2 of final state W-
  Double_t       fCosThetaFP;     // cos(theta_fdb) in W+ frame
  Double_t       fPhiFP;          // phi_fdb        in W+ frame
  Double_t       fXQ2WP;          // q^2 of final state W+

  ClassDef(WWBases, 1) // Bases for aa -> ww process
};

class WWSpring;

//_______________________________________________________________________
// =====================
//  class WWSpringBuf
// =====================
//-----------------------------------------------------------------------
class WWSpringBuf : public JSFSpringBuf {
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  WWSpringBuf(const char *name   = "WWSpringBuf", 
              const char *title  = "WW Spring test event buffer",
	      WWSpring *spring = 0)
	     : JSFSpringBuf(name,title,(JSFSpring*)spring) {}
  virtual ~WWSpringBuf() {}

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetQ2WW      ()           const { return fQ2WW;       }
  Double_t GetCosTheta  ()           const { return fCosTheta;   }
  Double_t GetPhi       ()           const { return fPhi;        }
  Double_t GetQ2WM      ()           const { return fQ2WM;       }
  Double_t GetCosThetaFM()           const { return fCosThetaFM; }
  Double_t GetPhiFM     ()           const { return fPhiFM;      }
  Double_t GetQ2WP      ()           const { return fQ2WP;       }
  Double_t GetCosThetaFP()           const { return fCosThetaFP; }
  Double_t GetPhiFP     ()           const { return fPhiFP;      }
  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual Bool_t SetPartons();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Event info
  // ----------------
  Double_t fQ2WW;           // q^2 of WW system
  Double_t fCosTheta;       // cos(theta_x) in cm  frame
  Double_t fPhi;            // phi_x        in cm  frame
  Double_t fQ2WM;           // q^2 of final state W-
  Double_t fCosThetaFM;     // cos(theta_fd)  in W- frame
  Double_t fPhiFM;          // phi_fd         in W- frame
  Double_t fQ2WP;           // q^2 of final state W+
  Double_t fCosThetaFP;     // cos(theta_fdb) in W+ frame
  Double_t fPhiFP;          // phi_fdb        in W+ frame
  Double_t fZBoost;         // p_z(cm)      in lab frame
  Double_t fEcmIP;          // Ecm

  ClassDef(WWSpringBuf, 1)  // WWSpring event buffer
};

//_______________________________________________________________________
// =====================
//  class WWSpring
// =====================
//-----------------------------------------------------------------------
class WWSpring : public JSFSpring {
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
public:
   WWSpring(const char *name  = "WWSpring",
            const char *title = "WW Spring test",
            WWBases  *bases = 0);
   virtual ~WWSpring();

   ClassDef(WWSpring, 1)  // WWSpring class
};
#endif
