#ifndef ZZSPRING_H
#define ZZSPRING_H
//*****************************************************************************
//* =====================
//*  ZZSpring
//* =====================
//*
//* (Description)
//*    aa -> ZZ generator
//*
//* (Update Record)
//*    2009/10/21  N.Maeda  Derived from WWSpring.
//*
//*****************************************************************************
#include <complex>
typedef std::complex<Double_t> Complex_t;

#include "TNamed.h"
#include "JSFModule.h"
#include "JSFBases.h"
#include "JSFSpring.h"

#include "GENLib.h"
#include "HELLib.h"

//_______________________________________________________________________
// =====================
//  class ZZBases
// =====================
//-----------------------------------------------------------------------
class ZZBases : public JSFBases {
friend class ZZSpringBuf;
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  ZZBases(const char *name  = "ZZBases", 
          const char *title = "ZZ Bases");
  virtual ~ZZBases();

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetEcm       ()           const { return fEcm;        }
  Double_t GetQ2ZZ      ()           const { return fQ2ZZ;       }
  Double_t GetCosTheta  ()           const { return fCosTheta;   }
  Double_t GetPhi       ()           const { return fPhi;        }
  Double_t GetQ2Z1      ()           const { return fQ2Z1;       }
  Double_t GetCosThetaF1()           const { return fCosThetaF1; }
  Double_t GetPhiF1     ()           const { return fPhiF1;      }
  Double_t GetQ2Z2      ()           const { return fQ2Z2;       }
  Double_t GetCosThetaF2()           const { return fCosThetaF2; }
  Double_t GetPhiF2     ()           const { return fPhiF2;      }
  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }

  void     SetEcmInit  (Double_t ecm       ) { fEcm     = ecm; }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual void     Userin();   // Bases user initialization
  virtual void     Userout();  // Bases user output 

  Double_t Func();     // Bases integration function.

  // ----------------------
  //   Utility methods
  // ----------------------
private:
  void      SelectHelicities(Double_t &weight);

  Double_t  DSigmaDX     (GENBranch &cmbranch);
  Complex_t AmpAAtoZZ    (Double_t betaz);
  void      TakeAmp      (Int_t fHelInitial[2], Double_t theta, Double_t betaz,
			  Complex_t FT[3][3]);


private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Job parameters
  // ----------------
  Double_t fEcm;            // Nominal Ecm
  Int_t    fZ1ModesLo;      // Z1 decay mode lo;
  Int_t    fZ1ModesHi;      // Z1 decay mode hi;
  Int_t    fZ2ModesLo;      // Z2 decay mode lo;
  Int_t    fZ2ModesHi;      // Z2 decay mode hi;

  // ----------------
  //  Particle Data
  // ----------------
  GENPDTZBoson *fZ1BosonPtr;      //! PD table entry of "Z1"
  GENPDTZBoson *fZ2BosonPtr;      //! PD table entry of "Z2"

  // ----------------
  //  Event info
  // ----------------
  Double_t       fZBoost;         // p_z(cm) in lab frame
  Double_t       fEcmIP;          // Ecm
  Double_t       fQ2ZZ;           // q^2 of ZZ system
  Double_t       fQ2Z1;           // q^2 of final state Z1
  Double_t       fQ2Z2;           // q^2 of final state Z2
  GENDecayMode  *fZ1ModePtr;      // pointer to Z1 decay mode
  GENPDTEntry   *f1Ptr;           // point to 1st Z1 daughter
  GENPDTEntry   *f2Ptr;           // point to 2nd Z1 daughter
  GENDecayMode  *fZ2ModePtr;      // pointer to Z2 decay mode
  GENPDTEntry   *f3Ptr;           // point to 1st Z2 daughter
  GENPDTEntry   *f4Ptr;           // point to 2nd Z2 daughter
  Int_t          fHelInitial[2];  // initial state helicities
  Int_t          fHelFinal  [4];  // final   state helicities
  Int_t          fJCombI;         // intial helicity combination
  Int_t          fJCombF;         // final  helicity combination
  Int_t          fZ1Mode;         // Z1 decay mode
  Int_t          fZ2Mode;         // Z2 decay mode
  ANL4DVector    fK[2];           // [0, 1] = [a, a]
  ANL4DVector    fP[4];           // [0, 1, 2, 3] = [f1, f2b, f3b, f4]
  Double_t       fM[4];           // [0,1,2,3] = [mf1, mf2b, mf3b, mf4]


  // ----------------
  //  Integ. vars
  // ----------------
  Double_t       fHelAA;          //! initial state helicity combination
  Double_t       fR_Lum1;         //! luminosity spectrum for photon 1
  Double_t       fR_Lum2;         //!                     for photon 2
  Double_t       fHelCombFinal;   // final   state helicity combination
  Double_t       fZ1DecayMode;    // decay mode selector for Z1
  Double_t       fZ2DecayMode;    // decay mode selector for Z2
  Double_t       fCosTheta;       // cos(theta_Z1)  in cm frame
  Double_t       fPhi;            // phi_Z1         in cm frame
  Double_t       fCosThetaF1;     // cos(theta_f1)  in Z1 frame
  Double_t       fPhiF1;          // phi_f1         in Z1 frame
  Double_t       fXQ2Z1;          // q^2 of final state Z1
  Double_t       fCosThetaF2;     // cos(theta_f3b) in Z2 frame
  Double_t       fPhiF2;          // phi_f3b        in Z2 frame
  Double_t       fXQ2Z2;          // q^2 of final state Z2

  ClassDef(ZZBases, 1) // Bases for aa -> zz process
};

class ZZSpring;

//_______________________________________________________________________
// =====================
//  class ZZSpringBuf
// =====================
//-----------------------------------------------------------------------
class ZZSpringBuf : public JSFSpringBuf {
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  ZZSpringBuf(const char *name   = "ZZSpringBuf", 
              const char *title  = "ZZ Spring test event buffer",
	      ZZSpring *spring = 0)
	     : JSFSpringBuf(name,title,(JSFSpring*)spring) {}
  virtual ~ZZSpringBuf() {}

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetQ2ZZ      ()           const { return fQ2ZZ;       }
  Double_t GetCosTheta  ()           const { return fCosTheta;   }
  Double_t GetPhi       ()           const { return fPhi;        }
  Double_t GetQ2Z1      ()           const { return fQ2Z1;       }
  Double_t GetCosThetaF1()           const { return fCosThetaF1; }
  Double_t GetPhiF1     ()           const { return fPhiF1;      }
  Double_t GetQ2Z2      ()           const { return fQ2Z2;       }
  Double_t GetCosThetaF2()           const { return fCosThetaF2; }
  Double_t GetPhiF2     ()           const { return fPhiF2;      }
  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }

  // ----------------------
  //   Base class methods
  // ----------------------
  virtual Bool_t SetPartons();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Event info
  // ----------------
  Double_t fQ2ZZ;           // q^2 of ZZ system
  Double_t fCosTheta;       // cos(theta_x) in cm  frame
  Double_t fPhi;            // phi_x        in cm  frame
  Double_t fQ2Z1;           // q^2 of final state Z1
  Double_t fCosThetaF1;     // cos(theta_f1)  in Z1 frame
  Double_t fPhiF1;          // phi_f1         in Z1 frame
  Double_t fQ2Z2;           // q^2 of final state Z2
  Double_t fCosThetaF2;     // cos(theta_f3b) in Z2 frame
  Double_t fPhiF2;          // phi_f3b        in Z2 frame
  Double_t fZBoost;         // p_z(cm)      in lab frame
  Double_t fEcmIP;          // Ecm

  ClassDef(ZZSpringBuf, 1)  // ZZSpring event buffer
};

//_______________________________________________________________________
// =====================
//  class ZZSpring
// =====================
//-----------------------------------------------------------------------
class ZZSpring : public JSFSpring {
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
public:
   ZZSpring(const char *name  = "ZZSpring",
            const char *title = "ZZ Spring test",
            ZZBases  *bases = 0);
   virtual ~ZZSpring();

   ClassDef(ZZSpring, 1)  // ZZSpring class
};
#endif
